/******************************************************
 *
 *	Control program for the PitSchu Rotary Encoder with stepper motor
 *	(c) Peter Schulten, Mlheim, Germany
 *	pit.schulten_(at)_arcor.de
 *
 *	Die unvernderte Wiedergabe und Verteilung dieses gesamten Sourcecodes
 *	in beliebiger Form ist gestattet, sofern obiger Hinweis erhalten bleibt.
 *
 * 	Ich stelle diesen Sourcecode kostenlos zur Verfgung und biete daher weder
 *	Support an noch garantiere ich fr seine Funktionsfhigkeit. Auerdem
 *	bernehme ich keine Haftung fr die Folgen seiner Nutzung.

 *	Der Sourcecode darf nur zu privaten Zwecken verwendet und modifiziert werden.
 *	Darber hinaus gehende Verwendung bedarf meiner Zustimmung.
 *
 *	History
 *	06.08.2014	PS		version 1 ready
 *
 */


#include "main.h"

#define EE_SIG1_ADDR			0xfe
#define EE_SIG_VAL				104
#define EE_VERSION_VAL			0x05

extern short		avgCycles;
extern short		smoothCycles;
extern short		hysteresis;
extern short		breakTime;
extern short		stepLimitL;
extern short		stepLimitR;

uint8_t				eeAddrIdx;

// list all vars to be saved here
eeByte eeValues[] = {
//	 EEaddr		pointer to var		size of var
	{0,		(void*)&avgCycles,				sizeof (avgCycles)},
	{2,		(void*)&smoothCycles,			sizeof(smoothCycles)},
	{4,		(void*)&hysteresis,				sizeof(hysteresis)},
	{6,		(void*)&breakTime,				sizeof(breakTime)},
	{8,		(void*)&stepLimitL,				sizeof(stepLimitL)},
	{10,	(void*)&stepLimitR,				sizeof(stepLimitR)},

	{0xff,	(uint8_t*)0,	 		0}		// end of list
	};

uint8_t epromCompare(uint8_t *p, uint8_t *vp, uint8_t size);
void epromRead(uint8_t *p, uint8_t *vp, uint8_t size);
void epromWrite(uint8_t *p, uint8_t *vp, uint8_t size);

//---------------------------------------------------------------------------------------------------


void eeInit (void)
{
	uint8_t		i;
	eeByte*		eP;

// check signature byte and version info. If mismatch then write default values to EEPROM

	if (eeprom_read_byte ((uint8_t*)EE_SIG1_ADDR) != EE_SIG_VAL ||
		eeprom_read_byte ((uint8_t*)EE_SIG1_ADDR+1) != EE_VERSION_VAL)
	{
		i = 0;
		eP = &eeValues[i];
		while (eP->addr != 0xff)
		{
			epromWrite ((uint8_t*)(0+eP->addr), eP->valP, eP->size);

			i++;
			eP = &eeValues[i];
		}

		i = EE_SIG_VAL;
		epromWrite ((uint8_t*)EE_SIG1_ADDR, &i, 1);
		i = EE_VERSION_VAL;
		epromWrite ((uint8_t*)(EE_SIG1_ADDR+1), &i, 1);
	}

	eeLoadSetup ();
}




void eeLoadSetup ()
{
	uint8_t i;
	eeByte*		eP;

	i = 0;
	eP = &eeValues[i];
	while (eP->addr != 0xff)
	{
		epromRead((uint8_t*)(0+eP->addr), eP->valP, eP->size);
		eP = &eeValues[++i];
	}
}



/*
 * write all bytes which where changed to EEPROM
 */
void eeUpdate (void)
{
	uint8_t i;
	eeByte*		eP;

	i = 0;
	eP = &eeValues[i];
	while (eP->addr != 0xff)
	{
		if (epromCompare(((uint8_t*)0 + eP->addr), eP->valP, eP->size) != 0)
			epromWrite ((uint8_t*)(0+eP->addr), eP->valP, eP->size);
		eP = &eeValues[++i];
	}
}




uint8_t epromCompare(uint8_t *p, uint8_t *vp, uint8_t size)
{
	uint8_t i = 0;

	while (i < size)
	{
		if (*vp++ != eeprom_read_byte (p++))
			return 1;
		i++;
	}

	return 0;
}



void epromRead(uint8_t *p, uint8_t *vp, uint8_t size)
{
	uint8_t i = 0;

	while (i < size)
	{
		*vp++ = eeprom_read_byte (p++);
		i++;
	}
}




void epromWrite(uint8_t *p, uint8_t *vp, uint8_t size)
{
	eeprom_busy_wait();
	cli();
	while (size)
	{
		eeprom_write_byte (p, (*vp & 0xff));
		eeprom_busy_wait();
		vp++;
		p++;
		size--;
	}
	sei();
}

